/*========================================================================
  Function Viewer: function module
  ---------------
  This program is distributed under the GPL license.

  Handles one user-defined function: Creates the textfield for writing the
function and its options, keeps the function parsed and ready to be
evaluated.
==========================================================================*/

#ifndef FUNCVIEWER_FUNCTION_HH
#define FUNCVIEWER_FUNCTION_HH

#include "fparser.hh"
#include <gtk/gtk.h>

class Function
{
 public:
    Function(unsigned functionNumber);
    ~Function();

    class Callback
    {
     public:
        virtual void functionHasChanged() = 0;
        virtual void functionIsBeingDestroyed(Function*) = 0;
    };

    void registerCallback(Callback*);

    inline bool shouldBeDrawn() const;
    inline bool isValid() const;
    inline FunctionParser& getParser();
    inline const GdkColor* getColor() const;

    inline GtkWidget* getWidget();





//-----------------------------------------------------------------------
 private:
//-----------------------------------------------------------------------
    GtkWidget* container;
    GtkWidget* checkButton;
    GtkWidget* textField;
    GdkColor functionColor;
    GtkWidget* colorSelectButton;

    Callback* callbackObject;

    bool validFunction;
    FunctionParser fParser;

    void textChanged();
    void checkButtonToggled();
    void colorChanged();

    static gboolean destroyCallback(GtkWidget*, gpointer);
    static void textModifyCallback(GtkEditable*, gpointer);
    static void buttonToggleCallback(GtkToggleButton*, gpointer);
    static void colorChangeCallback(GtkColorButton*, gpointer);

    Function(const Function&);
    Function& operator=(const Function&);
};

inline bool Function::shouldBeDrawn() const
{ return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkButton)); }

inline bool Function::isValid() const { return validFunction; }
inline FunctionParser& Function::getParser() { return fParser; }
inline const GdkColor* Function::getColor() const { return &functionColor; }
inline GtkWidget* Function::getWidget() { return container; }

#endif
